import re

from insights.contrib import ipaddress
from insights.parsers.ip import IpAddr, RouteDevices, IpNeighShow
from insights.parsers.hosts import Hosts
from insights.parsers.iptables import (
    IPTabPermanent,
    IP6TabPermanent,
    IPTables,
    IP6Tables,
)
from insights.parsers.netstat import Netstat
from insights.core.plugins import combiner

from insights_net.plugins.commands import CommandMetaClass, command
from insights_net.plugins.parsers.ip import NetNsIpAddr, NetNsIpAddr, NetNsIpRoute

from insights_net.plugins.combiners.ovn import OVNNB, OVNSB
from insights_net.plugins.combiners.ovs import OVS, OVSOfctlDump
from insights_net.plugins.parsers.ocp import OCPPods, OCPServices, OCPNetConf
from insights_net.plugins.parsers.ocp_net import OCPOfclDumpFlows


@combiner(
    optional=[
        IpAddr,
        RouteDevices,
        IpNeighShow,
        Hosts,
        IPTabPermanent,
        IP6TabPermanent,
        IP6Tables,
        IPTables,
        Netstat,
        NetNsIpAddr,
        NetNsIpRoute,
        OVSOfctlDump,
        OCPOfclDumpFlows,
        OVNNB,
        OVNSB,
        OVS,
        OCPPods,
        OCPServices,
        OCPNetConf,
    ]
)
class IPAddressInformation(metaclass=CommandMetaClass):
    """FindIP is a combiner that captures all other components that might hold
    ip addreesses and offers a function "find" that accepts an ip address
    string to look for.

    """

    def __init__(
        self,
        ipaddr,
        iproute,
        ipneigh,
        hosts,
        iptperm,
        ip6tperm,
        ip6tables,
        iptables,
        netstat,
        nsipaddr,
        nsiproute,
        ofctl,
        ocp_ofctl,
        ovn_nb,
        ovn_sb,
        ovs,
        pods,
        services,
        ocpnetconf,
    ):
        self.ipaddr = ipaddr
        self.iproute = iproute
        self.ipneigh = ipneigh
        self.hosts = hosts
        self.iptperm = iptperm
        self.ip6tperm = ip6tperm
        self.ip6tables = iptables
        self.iptables = iptables
        self.netstat = netstat
        self.nsipaddr = nsipaddr
        self.nsiproute = nsiproute
        self.ofctl = ofctl
        self.ocp_ofctl = ocp_ofctl
        self.ovn_nb = ovn_nb
        self.ovn_sb = ovn_sb
        self.ovs = ovs
        self.pods = pods
        self.services = services
        self.ocpnetconf = ocpnetconf

    @command
    def find_ip(self, ip):
        """Look for an IP address in all the available components"""

        ip_addr = ipaddress.ip_address(ip)
        result = dict()

        # Find in ip addr
        allip = [self.ipaddr]
        if self.nsipaddr:
            for ns in self.nsipaddr:
                allip.append(ns)

        ipaddr_matches = find_in_ipaddrs(ip_addr, allip)
        if ipaddr_matches:
            result["ip_addr"] = ipaddr_matches

        # Find in ip route
        allroute = [self.iproute]
        if self.nsiproute:
            for nsr in self.nsiproute:
                allroute.append(nsr)

        route_matches = find_in_routes(ip_addr, allroute)
        if route_matches:
            result["ip_route"] = route_matches

        # Find in neigh
        neigh_matches = find_in_neigh(ip_addr, self.ipneigh)
        if neigh_matches:
            result["ip_neigh"] = neigh_matches

        # Find in hosts
        host_matches = find_in_hosts(ip_addr, self.hosts)
        if host_matches:
            result["hosts"] = host_matches

        # Find in iptables
        ipt_matches = find_in_iptables(
            ip_addr, [self.iptperm, self.ip6tperm, self.ip6tables, self.iptables]
        )
        if ipt_matches:
            result["iptables"] = ipt_matches

        # Find in netstat
        netstat_matches = find_in_netstat(ip_addr, self.netstat)
        if netstat_matches:
            result["netstat"] = netstat_matches

        # Find in ovs ofctl dumps
        ofctl_matches = find_in_ofctl(ip_addr, self.ofctl or self.ocp_ofctl)
        if ofctl_matches:
            result["ofctl"] = ofctl_matches

        # Find in OVN
        ovn_nb_matches = find_in_nb(ip_addr, self.ovn_nb)
        if ovn_nb_matches:
            result["nb"] = ovn_nb_matches

        ovn_sb_matches = find_in_sb(ip_addr, self.ovn_sb)
        if ovn_sb_matches:
            result["sb"] = ovn_sb_matches

        # Find in OVS
        ovs_matches = find_in_ovs(ip_addr, self.ovs)
        if ovs_matches:
            result["ovs"] = ovs_matches

        # Find in OCP
        pods_matches = find_in_pods(ip_addr, self.pods)
        if pods_matches:
            result["pods"] = pods_matches

        services_matches = find_in_services(ip_addr, self.services)
        if services_matches:
            result["services"] = services_matches

        ocp_net_matches = find_in_ocpnetconf(ip_addr, self.ocpnetconf)
        if ocp_net_matches:
            result["ocp_net"] = ocp_net_matches

        return result


def find_in_ipaddrs(addr, ipaddr_parsers):
    """
    Finds an ip address in IPAddr objects
    Arg:
        addr: IpAddr object
        ipaddr_parsers: list of IPAddr Parsers
    """
    result = dict()
    for ipd in ipaddr_parsers:
        if not ipd:
            continue
        match = find_in_addrs(addr, ipd)
        if match:
            netns = ipd.netns if hasattr(ipd, "netns") else "default"
            result[netns] = match

    return result


def find_in_addrs(addr, ipdata):
    """
    Finds the ip address in the IpAddr parser
    Args:
        addr: IPAddr object
        ipdata: and instance of insights.parser.IpAddr
    """
    matches = []
    if not ipdata:
        return matches

    for iname, idata in ipdata.data.items():
        if any([addr == ipiface.ip for ipiface in idata.addresses]):
            matches.append(idata.data)

    return matches


def find_in_routes(addr, route_data):
    """
    Find the ip address in IPRouteDevice instances
    Addr:
        addr: IpAddr object
        route_data: list of IPRouteDevices
    """
    result = {}
    for route in route_data:
        match = _find_in_routes(addr, route)
        if match:
            netns = route.netns if hasattr(route, "netns") else "default"
            result[netns] = match

    return result


def _find_in_routes(addr, rt):
    """
    Finds the ip address in the RouteDevices parser
    """
    matches = []
    if not rt:
        return matches

    for match, route in rt.data.items():
        if match == "default":
            continue

        if _compare_ip_or_net(addr, match):
            matches.append({"match": match, "routes": [r.__dict__ for r in route]})

    if len(matches) == 0 and rt.data.get("default"):
        matches.append(
            {"match": "default", "routes": [r.__dict__ for r in rt.data.get("default")]}
        )

    return matches


def find_in_neigh(addr, neigh):
    """
    Find in IPNeighShow parser
    """
    matches = []
    if not neigh:
        return matches

    for neigh, data in neigh.data.items():
        if data.get("addr") == addr:
            append_data = data.copy()
            append_data["addr"] = neigh
            matches.append(append_data)

    return matches


def find_in_hosts(addr, host_parser):
    """
    Find in Hosts parser
    """
    matches = []
    if not host_parser:
        return matches

    for host_ip, hosts in host_parser.data.items():
        ip = ipaddress.ip_address(host_ip)
        if addr == ip:
            matches.extend(hosts)

    return matches


def find_in_iptables(addr, ipt_parsers):
    """
    Find an ipaddress in a list of insights.parser.IPTAblesConfiguration objects
    Args:
        addr: an IPAddress object
        ipt_parsers: list of insights.parser.IPTAblesConfiguration objects

    """
    matches = {}
    for data_source in ipt_parsers:
        if data_source is not None:
            match = find_in_iptables_common(addr, data_source)
            matches[data_source.__class__.__name__] = match

    return matches


def find_in_iptables_common(addr, ipt):
    """
    Only filter and nat tables supported
    ip: IPAddress object
    ipt: insights.parser.IPTAblesConfiguration
    """
    matches = []

    tables = ["filter", "nat"]
    for table in tables:
        for chain, rules in ipt.table_chains(table).items():
            for rule in rules:
                if find_in_rule(addr, rule["rule"]):
                    matches.append(
                        {
                            "chain": chain,
                            "rule": rule,
                        }
                    )
    return matches


def find_in_rule(addr, rule):
    """
    Looks for the ipAddres object (IPv4Address or IPv6Address) in the iptables
    rule string.
    Current rule matching supports:
        -s {IP}[/mask]
        -d {IP}[/mask]
        --to-destination {IP}[:{port}[-{port}]]
        --to-source {IP}[:{port}[-{port}]]

    Maybe this functionality could go in the nftables parser?
    """
    matches = [
        {
            "regexp": re.compile("-s\s([\w.:/]*)\s"),
        },
        {
            "regexp": re.compile("-d\s([\w.:/]*)\s"),
        },
        {
            "regexp": re.compile("--to-source\s([\w.:]*)\s"),
        },
        {
            "regexp": re.compile("--to-destination\s([\w.:]*)\s"),
        },
        {
            "regexp": re.compile("--to-source\s([\w.:]*):\d+\s"),
        },
        {
            "regexp": re.compile("--to-destination\s([\w.:]*):\d+\s"),
        },
        {
            "regexp": re.compile("--to-source\s([\w.:]*):\d+-\d+\s"),
        },
        {
            "regexp": re.compile("--to-destination\s([\w.:]*):\d+-\d+\s"),
        },
    ]

    for match in matches:
        result = match.get("regexp").search(rule)
        if result:
            if _compare_ip_or_net(addr, result.group(1)):
                return True

    return False


def _compare_ip_or_net(addr, match):
    """
    Tries to compare match string with an ipaddress object
    The match string can be an IP address or a subnet
    """
    try:
        net = ipaddress.ip_network(match)
        if addr in net:
            return True
    except ValueError:
        pass
    try:
        ip = ipaddress.ip_address(match)
        if addr == ip:
            return True
    except ValueError:
        pass

    return False


def find_in_netstat(addr, netstat):
    """
    Find ip address in a Netstat Parsers object
    Returns a dict keyed by Local or Foreign dependin on what column the match
    was made, e.g:
        {
        "Local": [{'Proto': 'tcp',
                   'Recv-Q': '0',
                   'Send-Q': '0',
                   'Local Address': '127.0.0.1:6633',
                   'Foreign Address': '0.0.0.0:*',
                   'State': 'LISTEN',
                   'User': '42435',
                   'Inode': '368485',
                   'PID/Program name': '61707/openvswitch-a',
                   'Timer': 'off (0.00/0/0)',
                   'PID': '61707',
                   'Program name': 'openvswitch-a',
                   'Local IP': '127.0.0.1',
                   'Port': '6633'},...],
        "Foreign":[...]
        }
    """
    ipcons = "Active Internet connections (servers and established)"
    if not netstat:
        return {}

    ns = netstat.datalist[ipcons]

    def compare_ip_port(addr, ip_port):
        """
        compares addr (IPAddress) with the ipport string {IP}:{Port}
        """
        ip, _, port = ip_port.rpartition(":")
        return addr == ipaddress.ip_address(ip)

    return {
        "Local": list(filter(lambda r: compare_ip_port(addr, r["Local Address"]), ns)),
        "Foreign": list(
            filter(lambda r: compare_ip_port(addr, r["Foreign Address"]), ns)
        ),
    }


def find_in_ofctl(addr, ofctls):
    """
    Finds in a list of plugin.OVSOfctlFlows dumps
    """
    if not ofctls:
        return

    fields = [
        "tun_src",
        "tun_dst",
        "tun_ipv6_src",
        "tun_ipv6_dst",
        "ct_nw_src",
        "ct_nw_dst",
        "ct_ipv6_src",
        "ct_ipv6_dst",
        "ip_src",
        "nw_src",
        "ip_dst",
        "nw_dst",
        "ipv6_src",
        "ipv6_dst",
        "arp_spa",
        "arp_tpa",
        "nd_target",
    ]
    base_actions = [
        "mod_nw_dst",
        "mod_nw_src",
    ]
    actions = base_actions
    actions.extend(["learn.{}".format(field) for field in fields])
    actions.extend(["clone.{}".format(act) for act in base_actions])
    actions.extend(["ct.nat.addrs.start", "ct.nat.addrs.end"])
    ## FIXME: Bundle and multipath missing

    all_fields = fields + actions

    expr = " or ".join(
        ["{field}~={ip}".format(field=field, ip=str(addr)) for field in all_fields]
    )

    result = {}
    for ofctl in ofctls:
        if not ofctl:
            continue
        flows = [str(f) for f in ofctl.flow_list.find(expr)]
        if flows:
            result[ofctl.bridge_name] = flows

    return result


# FIXME:
# - Only regexp on fields that make sense
# - IPv6 regex
IP_CIDR_RE = re.compile(r"((?<!\d\.)(?<!\d)(?:\d{1,3}\.){3}\d{1,3}(?:/\d{1,2})?)")


def list_exact(addr, addr_list):
    if not addr_list:
        return False

    for string in addr_list:
        for elem in string.split(" "):
            if _compare_ip_or_net(addr, elem):
                return True

    for string in addr_list:
        for elem in string.split(","):
            if _compare_ip_or_net(addr, elem):
                return True
    return False


def dict_regexp(addr, str_dict):
    """
    FIXME: IPV6
    """
    if not str_dict:
        return False

    for key, value in str_dict.items():
        if regexp_value(addr, value):
            return True

    return False


def regexp_value(addr, value):
    if not isinstance(value, str):
        return False

    for match in IP_CIDR_RE.findall(value):
        if _compare_ip_or_net(addr, match):
            return True
    return False


def exact(addr, string):
    return _compare_ip_or_net(addr, string)


NBFIELDS = {
    "Address_Set": {"addresses": list_exact},
    "Logical_Switch_Port": {
        "addresses": list_exact,
        "dynamic_addresses": list_exact,
        "external_ids": dict_regexp,
    },
    "DHCP_Options": {
        "options": dict_regexp,
        "external_ids": dict_regexp,
        "cidr": exact,
    },
}

SBFIELDS = {
    "Address_Set": {"addresses": list_exact},
    "Encap": {
        "ip": exact,
    },
    "IGMP_Group": {"address": exact},
    "Logical_Flow": {"match": regexp_value, "actions": regexp_value},
}

OVSFIELDS = {
    "Open_vSwitch": {"external_ids": dict_regexp},
    "Interface": {"options": dict_regexp},
    "Port": {"external_ids": dict_regexp},
    "Manager": {"target": regexp_value},
}


def find_in_nb(addr, ovndb):
    if not ovndb:
        return
    if isinstance(ovndb, list):
        ovndb = ovndb[0]
    return find_in_ovsdb(addr, ovndb, NBFIELDS)


def find_in_sb(addr, ovndb):
    if not ovndb:
        return
    if isinstance(ovndb, list):
        ovndb = ovndb[0]
    return find_in_ovsdb(addr, ovndb, SBFIELDS)


def find_in_ovs(addr, ovsdb):
    if not ovsdb:
        return
    if isinstance(ovsdb, list):
        ovsdb = ovsdb[0]
    return find_in_ovsdb(addr, ovsdb, OVSFIELDS)


def find_in_ovsdb(addr, ovsdb, fields):
    result = {}
    for name, matches in fields.items():
        table = ovsdb.table(name)
        if not table:
            continue
        for uid, row in table.items():
            for column, match in fields.get(name).items():
                if match(addr, row.get(column)):
                    if not result.get(name):
                        result[name] = []
                    result[name].append(row)

    return result


def find_in_pods(addr, pod_data):
    return find_in_ocp_namespaces(addr, pod_data, "status", "podIP")


def find_in_services(addr, service_data):
    return find_in_ocp_namespaces(addr, service_data, "spec", "clusterIPs")


def find_in_ocp_namespaces(addr, ocp_data, *field_list):
    result = []
    if not ocp_data:
        return

    for resource_list in ocp_data:
        for item in resource_list.get("items"):
            match = find_in_ocp(addr, item, *field_list)
            if match:
                match["namespace"] = resource_list.namespace
                result.append(match)
    return result


def find_in_ocpnetconf(addr, ocp_net):
    result = []
    if not ocp_net:
        return

    for item in ocp_net.get("items"):
        match = find_in_ocp(addr, item, "spec", "clusterNetwork")
        if match:
            result.append(match)
        match = find_in_ocp(addr, item, "spec", "serviceNetwork")
        if match:
            result.append(match)
    return result


def find_in_ocp(addr, item, *field_list):
    """
    Finds an IP address in a yaml parser item
    Args:
        addr (IpAddr): the address to look for
        item (YAMLParser): the item to look nto
        *field_list (list(str)): a list of fields used to index the item
    """
    # Navigate to the specific field
    elem = item
    for f in field_list:
        elem = elem.get(f)
        if not elem:
            return None

    match, subfield = find_in_field(addr, elem)

    if match:
        return {
            "name": item.get("metadata").get("name")
            if item.get("metadata")
            else "unknown",
            "full": item,
            "field": field_list[-1],
            "match": elem,
        }
    else:
        return None


def find_in_field(addr, field):
    """
    Looks for an IP address in a field that can be a string, list or a dictionary
    Args:
        addr(IpAddr): the address to look for
        field (dict or str): the field to look into
    Returns:
        (bool, optional(string)): a tuple. First value, if matched. Second, if field is
        a dictionary, the key where it matched
    """
    if isinstance(field, dict):
        for k, v in field.items():
            if v and _compare_ip_or_net(addr, v):
                return (True, k)
    elif isinstance(field, list):
        for elem in field:
            match, subfield = find_in_field(addr, elem)
            if match:
                return (True, subfield)
    elif isinstance(field, str):
        if field and _compare_ip_or_net(addr, field):
            return (True, None)

    return False, None



"""
Plugin that deals with must-gather gather_network_logs content
"""

from insights.core.plugins import parser
from insights.parsers import SkipException
from insights.core.spec_factory import glob_file, RawFileProvider

from insights_net.plugins.datasources import GZFileProvider
from insights_net.plugins.parsers.ovsdb import OVSDBDumpParser
from insights_net.plugins.parsers.ovs import OVSOfctlDumpBase


"""
OVN Databases from network_logs/ovnkube*{nb,sb}db.gz
"""
# Cannot put MustGatherContext because it fails to detect the context if
# we have gather_network_logs
ocp_nb = glob_file("*/network_logs/ovnkube-*_nbdb.gz", kind=GZFileProvider)
ocp_sb = glob_file("*/network_logs/ovnkube-*_sbdb.gz", kind=GZFileProvider)


@parser(ocp_nb)
class OCPNB(OVSDBDumpParser):
    def __init__(self, *args, **kwargs):
        super(OCPNB, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        self.pod_name = self.file_name.rpartition("_nbdb.gz")
        return super(OCPNB, self).parse_content(content)


@parser(ocp_sb)
class OCPSB(OVSDBDumpParser):
    def __init__(self, *args, **kwargs):
        super(OCPSB, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        self.pod_name = self.file_name.rpartition("_sbdb.gz")
        return super(OCPSB, self).parse_content(content)


"""
Ofproto dumps
"""
ocp_flows = glob_file("*/network_logs/*ofctl_dump_flows*")


@parser(ocp_flows)
class OCPOfclDumpFlows(OVSOfctlDumpBase):
    def __init__(self, *args, **kwargs):
        super(OCPOfclDumpFlows, self).__init__(*args, **kwargs)

    @property
    def hostname(self):
        return self._hostname

    def parse_content(self, content):
        if not content:
            raise SkipException("Empty Content")

        # Extract the bridge name
        try:
            parts = self.file_name.split("_ovs_ofctl_dump_flows_")
            self._hostname = parts[0]
            self._bridge_name = parts[1]
        except:
            raise SkipException("Invalid Path!")

        return super(OCPOfclDumpFlows, self).parse_content(content)



import click
from rich.console import Console
from rich.columns import Columns

from insights_net.main import maincli
from insights_net.commands.printing import print_section_header, print_archive_header


@maincli.command(name="info")
@click.pass_obj
def info(ctx):
    """
    Show basic information of the archives
    """
    data = ctx.client.available()
    if not data:
        print("Command not found")
        return

    console = Console()
    for archive, host_data in data.items():
        if not host_data:
            continue

        print_archive_header(console, "Archive: " + archive)
        if isinstance(host_data, str):
            console.print(host_data)
        else:
            print_results(console, host_data)
            print("")


def print_results(console, data):
    if data.get("models"):
        print_section_header(console, "Available Models")
        models = data.get("models")
        console.print(Columns(models, equal=True, expand=True))
        console.print("")

    if data.get("commands"):
        print_section_header(console, "Available Commands")
        for cmd in data.get("commands"):
            console.print("  {}".format(cmd))



from rich.emoji import Emoji
from rich.panel import Panel
from rich.pretty import Pretty
from rich.table import Table
from rich.text import Text


def print_section_header(console, text):
    console.print("* " + text, style="bold")
    console.print("  " + "-" * len(text), style="bold")


def print_archive_header(console, name):
    console.print(
        Panel(
            Text(
                Emoji.replace(":scroll:")
                + " Archive: %s " % name
                + Emoji.replace(":scroll:"),
                style="bold",
                justify="center",
            )
        )
    )


def print_table(console, data, title, tables_as_lists=False):
    """
    Prints a table content
    Args:
        console (rich.Console) where text is to be printed
        data (list(dict)): list of dictionaries of table items.
            Columns are the keys of the elements
    """
    if not data:
        return

    if tables_as_lists:
        for row in data:
            console.print(row)
        return
    else:
        tt = Table(title=title)
        for header, value in data[0].items():
            if isinstance(value, dict):
                tt.add_column(
                    header, justify="left", no_wrap=False, ratio=1, min_width=25
                )
            else:
                tt.add_column(
                    header, justify="left", no_wrap=False, min_width=len(header)
                )

        for elem in data:
            values = [Pretty(val) for val in elem.values()]
            tt.add_row(*values)

        console.print(tt)



import glob
import uuid
import json
import logging
import os
import os.path
import stat

from insights import CommandParser, parser, Parser, datasource
from insights.core import dr
from insights.core.context import SosArchiveContext, HostArchiveContext
from insights.core.spec_factory import simple_file, ContentProvider
from insights.parsers import SkipException

from ovsdbapp.backend.ovs_idl import Backend
from ovsdbapp.backend.ovs_idl import connection

log = logging.getLogger(__name__)


class OVSDBMixin(object):
    """
    Base class for OVSDB data parsers and combiners.

    Subclasses must populate two attributes:
        _name(str): the name of the database
        _tables(dict): Must have the following format:
            {
                "TableName0": {
                    "uuid0": {
                        "column1": value01,
                        "column2": value02,
                        ...
                    }
                    "uuid1": {
                        "column1": value11,
                        "column2": value12,
                    }
                    ...
                }
                "TableName1": {
                ...
                }
            }
    """

    @property
    def tables(self):
        """
        Returns all the tables
        """
        return self._tables

    @property
    def name(self):
        """
        Returns the database name
        """
        return self._name

    def table_list(self):
        """
        Returns all the tables
        """
        return list(self._tables.keys())

    def columns(self, table):
        """
        Returns the list columns in a particular table
        """
        first = self._tables.get(next(list(self._tables.get(table).keys())))
        return first.keys()

    def table(self, name):
        """
        Returns the table with the given name
        """
        return self._tables.get(name)

    def row(self, table, uuid):
        """
        Finds the row that with the given uuid
        """
        table = self._tables.get(table)
        if table:
            return table.get(uuid)
        return None

    def find_uuid(self, table, uuid):
        """
        Finds the row that in the table that matches the column's value
        """
        return self.filter(table, lambda x: x.get("_uuid").startswith(uuid))

    def find(self, table, column, value):
        """
        Finds the row that in the table that matches the column's value
        """
        table = self._tables.get(table)
        if table:
            return list((row for row in table.values() if row.get(column) == value))
        return []

    def filter(self, table, function):
        """
        Filters the table based on the given callable filter
        """
        table = self._tables.get(table)
        if table:
            return list(filter(function, table.values()))
        return []


class OVSDBParser(OVSDBMixin, Parser):
    """
    Base class for Parsers that implement OVSDB instances
    """

    def __init__(self, *args, **kwargs):
        super(OVSDBParser, self).__init__(*args, **kwargs)


class OVSDBListParser(OVSDBParser):
    """
    OVSDBListParser tries to parse the output of commands that use the "--list"
    format. Examples: ovs-vsctl, ovn-nbcl, ovn-sbctl

    Input example:
    ==============
    AutoAttach table

    Bridge table
    _uuid               : 45711587-9496-4d14-8dfe-67a8c273e610
    auto_attach         : []
    controller          : [65c7f516-5bef-41f1-b633-94d6375fa356]
    datapath_id         : "000048df37ce6d70"
    datapath_type       : system
    datapath_version    : "<unknown>"
    external_ids        : {bridge-id=br-link}
    fail_mode           : secure
    flood_vlans         : []
    flow_tables         : {}
    ipfix               : []
    mcast_snooping_enable: false
    mirrors             : []
    name                : br-link
    netflow             : []
    other_config        : {mac-table-size="50000"}
    ports               : [8918bf51-abfa-4b2e-961f-2dca931bf2a8, acb08497-7deb-4ca7-bc9d-d1119dc2d85f, dcf4d708-b4d8-4948-8570-d2a6a05e7949]
    """

    def __init__(self, *args, **kwargs):
        super(OVSDBListParser, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        self._tables = dict()
        current_table = dict()
        current_table_name = ""
        current_uuid = ""
        current_row = dict()
        for line in content:
            line = line.strip()
            if not line:
                continue

            table_name, _, keyword = line.partition(" ")
            if keyword == "table":
                # New Table, save old row and table
                if current_uuid != "":
                    current_table[current_row["_uuid"]] = current_row
                    current_uuid = ""

                if current_table_name != "":
                    self._tables[current_table_name] = current_table

                current_table_name = table_name
                current_table = dict()

            else:
                column, _, value = line.partition(":")
                column = column.strip()
                value = value.strip()
                converted = self._convert_value(value)

                if not value or not column:
                    raise SkipException("Wrong format")

                if column == "_uuid":
                    # New rowect, save old row if any
                    if current_uuid != "":
                        current_table[current_row["_uuid"]] = current_row

                    current_uuid = converted
                    current_row = dict()

                current_row[column] = converted

    def _convert_dict(self, value):
        """
        Convert a dictionary field value
        e.g:
        {classless_static_route="{169.254.169.254/32,192.168.199.2, 0.0.0.0/0,192.168.199.1}", dns_server="{172.16.0.1, 10.0.0.1}", domain_name="\"openstackgate.local\"", lease_time="43200", mtu="1442", router="192.168.199.1", server_id="192.168.199.1", server_mac="fa:16:3e:0b:ba:9d"}
        """
        dict_value = value.strip("{")
        result = {}
        while True:
            (key, found, rest) = dict_value.partition("=")
            if not found:
                break

            key = key.strip('"')

            item = None
            new_dict_value = ""

            if rest[0] == '"':
                start = 1
                while True:
                    pos = rest.find('"', start)
                    if pos == len(rest):
                        # Error, we did not find the end of the string
                        raise Exception("Wrong format %s" % value)

                    if rest[pos - 1] != "\\":
                        # found a non escaped "
                        item = rest[1:pos]
                        new_dict_value = rest[pos + 1 :].strip(", ")
                        break

                    start = pos + 1
            else:
                (item, comma, new_dict_value) = rest.partition(", ")
                if not comma:
                    (item, curly, new_dict_value) = rest.partition("}")
                    if not curly:
                        raise Exception("Wrong format %s" % value)

            result[key] = self._convert_single_value(item)
            dict_value = new_dict_value

        return result

    def _convert_value(self, value):
        if value[0] == "[":
            converted = []
            for val in value.strip("[]").split(", "):
                if val:
                    converted.append(self._convert_single_value(val))
            return converted
        elif value[0] == "{":
            converted = self._convert_dict(value)
        else:
            converted = self._convert_single_value(value)

        return converted

    def _convert_single_value(self, value):
        value = value.strip('\\"')
        try:
            int_val = int(value)
            return int_val
        except ValueError:
            pass

        try:
            float_val = float(value)
            return float_val
        except ValueError:
            pass

        if value == "true":
            return True
        elif value == "false":
            return False

        return value


class OVSDBDumpParser(OVSDBParser):
    """
    A parser that reads a OVSDB Dump file.

    Note that as OVSDB is transactional, if the dump has not been compacted,
    some information will be lost.

    Only the fist database will be parsed. Further databases or transactions will be ignored
    """

    def parse_content(self, content):
        if len(content) < 2:
            # raise SkipException("Wrong format")
            raise Exception("Wrong format")

        # First line should be OVSDB CLUSTER {} {}
        header = content[0]
        if header[0:13] != "OVSDB CLUSTER":
            # raise SkipException("Wrong format")
            raise Exception("Wrong format: {} ".format(header))

        dump = json.loads(content[1])
        name = dump.get("name")
        if not name:
            # raise SkipException("Wrong format")
            raise Exception("Wrong format")

        prev_data = dump.get("prev_data")
        if not prev_data or len(prev_data) != 2:
            # raise SkipException("Wrong format")
            raise Exception("Wrong format")

        self._tables = prev_data[1]
        for table_name, table in self._tables.items():
            if not isinstance(table, dict):
                continue
            for uuid, row in table.items():
                for column, data in row.items():
                    # in place replacement
                    row[column] = self.process_single_field(data)

    def process_single_field(self, data):
        """
        Process a single field that can be a map, set or uuid
        """
        if isinstance(data, list) and len(data) > 1:
            if data[0] == "set":
                return data[1]
            elif data[0] == "map":
                return {item[0]: self.process_single_field(item[1]) for item in data[1]}
            elif data[0] == "uuid":
                return data[1]
            else:
                return data
        else:
            return data


class ovsdb_servers(object):
    """For each valid OVSDB connection found, creates a datasource that
    connects to such server and dumps all its content into a dictionary

    Metadata:
        extra_patters: extra patterns can be provided as metadata

    Args:
        patterns (str or [str]): file patterns of unix sockets
        schema

    """

    def __init__(self, patterns, schema, context=None, deps=[], **kwargs):
        if not isinstance(patterns, (list, set)):
            patterns = [patterns]
        self.patterns = patterns

        self.schema = schema
        self.context = context or HostArchiveContext
        self.__name__ = self.__class__.__name__
        datasource(self.context, *deps, **kwargs)(self)

    def __call__(self, broker):
        """
        Called by the plugin to extract the data

        Find all the matched patterns and return the data
        """
        if isinstance(self.context, list):
            ctx = dr.first_of(self.context, broker)
        else:
            ctx = broker.get(self.context)

        results = []
        root = ctx.root

        for pattern in self.patterns:
            pattern = ctx.locate_path(pattern)
            for path in sorted(glob.glob(os.path.join(root, pattern.lstrip("/")))):
                if os.path.isdir(path) or not stat.S_ISSOCK(os.stat(path).st_mode):
                    continue
                log.debug("ovsdb socket found at %s" % path)

                try:
                    client = OVSDBClient(path, self.schema)
                except Exception as e:
                    log.debug(
                        "ovsdb does not contain a database with schema %s", self.schema
                    )
                    continue
                results.append(client)

        return results


class OVSDBClient:
    """OVSDBClient is a Datasource that connects to a running ovsdb-server,
    dumps all its tables. Unlike other Datasources, OVSDBClient is does not
    expose its content as a list of strings, but as a dictionary of tables.
    This is because the internal use of ovsdbapp who already parses the content

    In order to be used by a Parser it has the following attributes:
        relative_path: the relative_path to the ovsdb socket that is used to
            interact with the ovsdb server
        path: full path of the socket file
        content: the dictionary of tables with the following format:
            {
                "TableName_1" {
                    "uuid_0" : {
                        "column_name_1": _value_,
                        "column_name_2": _value_
                        ...
                    }
                    "uuid_1" : {
                        "column_name_1": _value_,
                        "column_name_2": _value_
                        ...
                    }
                    ...
                }
                "TableName_2" {
                ...
                }
            }

    Args:
        relative_path: relative_path to the db socket file
        schema: the database name which shall be dumped

    Raises:
        Exception if the database is not pressent in the ovsdb-server
        instance
    """

    def __init__(self, relative_path, schema, root="/"):
        self._relative_path = relative_path
        self._root = root
        self.idl = connection.OvsdbIdl.from_server("unix:" + self.path, schema)
        self.conn = connection.Connection(idl=self.idl, timeout=3)
        self.api = Backend(self.conn)
        self._name = schema
        self._content = None
        self._exception = None

    @property
    def path(self):
        return os.path.join(self._root, self.relative_path)

    @property
    def relative_path(self):
        return self._relative_path

    @property
    def content(self):
        if self._exception:
            raise self._exception

        if self._content is None:
            try:
                self._content = self._dump_tables()
            except Exception as ex:
                self._exception = ex
                raise

        return self._content

    def _dump_tables(self):
        result = {}
        for table, rows in self.api.idl.tables.items():
            # Extract rows and placed them as
            # "tableName" : {
            #    "uuid" : { rowData }
            # }

            result[table] = {
                str(row["_uuid"]): self._process_row(row)
                for row in self.api.db_list(table).execute()
            }
        return result

    def _process_row(self, row):
        """Ensure all of items are native types.
        In practice this just means converting UUIDs to strings
        """
        for col, value in row.items():
            if (
                isinstance(value, list)
                and len(value) > 0
                and isinstance(value[0], uuid.UUID)
            ):
                row[col] = [str(uuid) for uuid in value]
            elif (
                isinstance(value, dict)
                and len(value) > 0
                and isinstance(list(value.values())[0], uuid.UUID)
            ):
                row[col] = {k: str(v) for k, v in value.items()}
            elif isinstance(value, uuid.UUID):
                row[col] = str(value)
        return row



from insights.core.context import MustGatherContext, ExecutionContext, FSRoots
from insights.core.plugins import datasource, parser
from insights.core.spec_factory import simple_file
from insights.parsers import SkipException
from insights import YAMLParser

from insights_net.plugins.datasources import recursive_dir


netconf = simple_file(
    "cluster-scoped-resources/config.openshift.io/networks.yaml",
    contexte=MustGatherContext,
)


@parser(netconf, context=MustGatherContext)
class OCPNetConf(YAMLParser):
    """
    Contains the OCP network configuration yaml
    """

    pass


class OCPNamespaceResource(YAMLParser):
    """
    Base class parser for the files in: "namespaces/$NAMESPACE/$RESOURCE_TYPE
    """

    def __init__(self, *args, **kwargs):
        super(OCPNamespaceResource, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        subpath = self.file_path.rpartition("namespaces/")
        if not subpath[2]:
            raise SkipException("Invalid Path: not a valid OCPNamespace resource")

        subpath = subpath[2]
        self.namespace = subpath.split("/")[0]
        self.resource_type = subpath.split("/")[1]
        self.resource = subpath.split("/")[2].split(".")[0]

        return super(OCPNamespaceResource, self).parse_content(content)


"""
Files in {namespace}/core/
"""
services = recursive_dir(
    "namespaces", include="core/services.yaml", context=MustGatherContext
)


@parser(services, context=MustGatherContext)
class OCPServices(OCPNamespaceResource):
    pass


pods = recursive_dir("namespaces", include="core/pods.yaml", context=MustGatherContext)


@parser(pods, context=MustGatherContext)
class OCPPods(OCPNamespaceResource):
    pass


configmaps = recursive_dir(
    "namespaces", include="core/configmaps.yaml", context=MustGatherContext
)


@parser(configmaps, context=MustGatherContext)
class OCPConfigMaps(OCPNamespaceResource):
    pass


events = recursive_dir(
    "namespaces", include="core/events.yaml", context=MustGatherContext
)


@parser(events, context=MustGatherContext)
class OCPEvents(OCPNamespaceResource):
    pass


endpoints = recursive_dir(
    "namespaces", include="core/endpoints.yaml", context=MustGatherContext
)


@parser(endpoints, context=MustGatherContext)
class OCPEndpoints(OCPNamespaceResource):
    pass


persistentvolumeclaims = recursive_dir(
    "namespaces", include="core/persistentvolumeclaims.yaml", context=MustGatherContext
)


@parser(persistentvolumeclaims, context=MustGatherContext)
class OCPPersistentVolumeClaims(OCPNamespaceResource):
    pass


replicationcontrollers = recursive_dir(
    "namespaces", include="core/replicationcontrollers.yaml", context=MustGatherContext
)


@parser(replicationcontrollers, context=MustGatherContext)
class OCPReplicationControllers(OCPNamespaceResource):
    pass


secrets = recursive_dir(
    "namespaces", include="core/secrets.yaml", context=MustGatherContext
)


@parser(secrets, context=MustGatherContext)
class OCPSecrets(OCPNamespaceResource):
    pass


"""
Files in {namespace}/apps/
"""
daemonsets = recursive_dir(
    "namespaces", include="apps/daemonsets.yaml", context=MustGatherContext
)


@parser(daemonsets, context=MustGatherContext)
class OCPDaemonsets(OCPNamespaceResource):
    pass


deployments = recursive_dir(
    "namespaces", include="apps/deployments.yaml", context=MustGatherContext
)


@parser(deployments, context=MustGatherContext)
class OCPDeployments(OCPNamespaceResource):
    pass


replicasets = recursive_dir(
    "namespaces", include="apps/replicasets.yaml", context=MustGatherContext
)


@parser(replicasets, context=MustGatherContext)
class OCPReplicaSets(OCPNamespaceResource):
    pass


statefulsets = recursive_dir(
    "namespaces", include="apps/statefulsets.yaml", context=MustGatherContext
)


@parser(statefulsets, context=MustGatherContext)
class OCPStatefulSets(OCPNamespaceResource):
    pass


routes = recursive_dir(
    "namespaces", include="route.openshift.io/routes.yaml", context=MustGatherContext
)


@parser(routes, context=MustGatherContext)
class OCPRoutes(OCPNamespaceResource):
    pass



# -*- coding: utf-8 -*-

import click

from insights_net.client import InsightsClient


class Context(object):
    def __init__(self, verbose=False):
        self.client = InsightsClient(verbose)
        self.verbose = verbose


@click.group()
@click.option("-v", "--verbose", is_flag=True, help="Be verbose")
@click.pass_context
def maincli(ctx, verbose):
    ctx.obj = Context(verbose=verbose)
    pass


@maincli.command(name="stop")
@click.pass_obj
def stop(obj):
    """
    Stop the background running insights kernel
    """
    obj.client.close()


def main():
    maincli()



import re
from insights import parser, CommandParser
from insights.core.context import SosArchiveContext
from insights.core.spec_factory import simple_file, glob_file
from insights.parsers import SkipException
from ovs_dbg.ofp import OFPFlow
from ovs_dbg.filter import OFFilter

from insights_net.plugins.parsers.ovsdb import (
    OVSDBParser,
    OVSDBListParser,
    ovsdb_servers,
)

ovsdb_dump = simple_file(
    "/sos_commands/openvswitch/ovsdb-client_-f_list_dump", context=SosArchiveContext
)


@parser(ovsdb_dump)
class OVSDump(OVSDBListParser):
    def __init__(self, *args, **kwargs):
        super(OVSDump, self).__init__(*args, **kwargs)
        self._name = "Open_vSwitch"


ovs_db_local = ovsdb_servers(["*.sock"], "Open_vSwitch")


@parser(ovs_db_local)
class OVSLocal(OVSDBParser):
    def __init__(self, *args, **kwargs):
        super(OVSLocal, self).__init__(*args, **kwargs)
        self._name = "OpenvSwitch"

    def parse_content(self, data):
        self._tables = data


# Openflow parsers
ofctl_dump_flows = glob_file(
    "/sos_commands/openvswitch/ovs-ofctl_dump-flows*", context=SosArchiveContext
)

ofctl_show = glob_file(
    "/sos_commands/openvswitch/ovs-ofctl_show_*", context=SosArchiveContext
)


class OVSOfctlDumpBase(CommandParser):
    def __init__(self, *args, **kwargs):
        self._flow_list = None
        self._bridge_name = None
        super(OVSOfctlDumpBase, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        if not content:
            raise SkipException("Empty Content!")

        try:
            self._flow_list = OVSFlowList(content)
        except Exception as e:
            raise SkipException("Failed to parse flows") from e

    @property
    def bridge_name(self):
        return self._bridge_name

    @property
    def flow_list(self):
        return self._flow_list


@parser(ofctl_dump_flows)
class OVSOfctlDump(OVSOfctlDumpBase):
    def __init__(self, *args, **kwargs):
        super(OVSOfctlDump, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        if not content:
            raise SkipException("Empty Content")
        # Extract the bridge name
        try:
            self._bridge_name = self.file_path.split("ovs-ofctl_dump-flows_")[1]
        except Exception:
            raise SkipException("Invalid Path!")

        return super(OVSOfctlDump, self).parse_content(content)


@parser(ofctl_show)
class OVSOfctlShow(CommandParser):
    """
    Input example:
    ==============
    OFPT_FEATURES_REPLY (xid=0x2): dpid:0000f613db047547
    n_tables:254, n_buffers:0
    capabilities: FLOW_STATS TABLE_STATS PORT_STATS QUEUE_STATS ARP_MATCH_IP
    actions: output enqueue set_vlan_vid set_vlan_pcp strip_vlan mod_dl_src mod_dl_dst mod_nw_src mod_nw_dst mod_nw_tos mod_tp_src mod_tp_dst
     1(int-br-link): addr:e2:c2:43:a8:6c:97
         config:     0
         state:      0
         speed: 0 Mbps now, 0 Mbps max
     2(int-br-ex): addr:da:34:23:5e:25:18
         config:     0
         state:      0
         speed: 0 Mbps now, 0 Mbps max
     LOCAL(br-int): addr:f6:13:db:04:75:47
         config:     PORT_DOWN
         state:      LINK_DOWN
         speed: 0 Mbps now, 0 Mbps max
    OFPT_GET_CONFIG_REPLY (xid=0x4): frags=normal miss_send_len=0
    """

    def __init__(self, *args, **kwargs):
        super(OVSOfctlShow, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        if not content:
            raise SkipException("Empty Content!")

        # Extract the bridge name
        try:
            self._bridge_name = self.file_path.split("ovs-ofctl_show_")[1]
        except:
            raise SkipException("Invalid Path!")

        # Parse table line, e.g:
        # n_tables:254, n_buffers:0
        n_tables, _, n_buffers = content[1].strip().partition(", ")
        self._tables = int(n_tables.split(":")[1])
        self._buffers = int(n_buffers.split(":")[1])

        # Parse capabilities, e.g:
        # capabilities: FLOW_STATS TABLE_STATS PORT_STATS QUEUE_STATS
        _, _, caps = content[2].strip().partition(": ")
        self._capabilities = caps.split(" ")

        # Parse actions, e.g:
        # actions: output enqueue set_vlan_vid set_vlan_pcp strip_vlan
        _, _, actions = content[3].strip().partition(": ")
        self._actions = actions.split(" ")

        # Parse ports
        port_re = re.compile("(\w+)\(([\w-]+)\): addr:([\w:]+)")
        self._ports = []
        port_idx = -1
        for line in content[4:-1]:
            line = line.strip()
            match = port_re.match(line)
            if match:
                num_str = match.group(1)
                num_int = 0  # represents LOCAL
                try:
                    num_int = int(num_str)
                except ValueError:
                    pass

                self._ports.append(
                    {
                        "num": num_int,
                        "name": match.group(2),
                        "addr": match.group(3),
                    }
                )
                port_idx += 1
            else:
                if port_idx < 0:
                    raise SkipException("Parsing error")

                keyword, _, info = line.partition(":")
                keyword = keyword.strip()
                info = info.strip()

                if keyword == "config":
                    self._ports[port_idx]["config"] = info
                elif keyword == "state":
                    self._ports[port_idx]["state"] = info
                elif keyword == "speed":
                    m_now, _, m_max = info.partition(", ")
                    self._ports[port_idx]["speed"] = {
                        "now": m_now.split("now")[0].strip(),
                        "max": m_max.split("max")[0].strip(),
                    }

    @property
    def bridge_name(self):
        """
        (str): Get the bridge name
        """
        return self._bridge_name

    def port(self, num):
        """
        (dict) or None: Get the port with matching num
        """
        for port in self._ports:
            if port["num"] == num:
                return port

        return None

    @property
    def ports(self):
        """
        (list): Returns the list of port dicts
        """
        return self._ports

    @property
    def capabilities(self):
        """
        (list): Returns the list of capabilities
        """
        return self._capabilities

    @property
    def actions(self):
        """
        (list): Returns the list of actions
        """
        return self._actions

    @property
    def tables(self):
        """
        (int): Returns the number of tables
        """
        return self._tables

    @property
    def buffers(self):
        """
        (int): Returns the number of buffers
        """
        return self._buffers


class OVSFlowList(object):
    """OVSFlowList represents a list of flows

    It parsers a list of strings and exposes some useful commands

    Args:
        flow_list(list(str)): list of strings with flows
    """

    def __init__(self, flow_list):
        self._flows = []
        for flow_string in flow_list:
            if " reply " in flow_string:
                continue
            self._flows.append(OFPFlow.from_string(flow_string))

    @property
    def flows(self):
        return self._flows

    @property
    def len(self):
        return len(self._flows)

    def find(self, expr):
        """
        Find flows that match expr.

        expr(str): an expression following ovs-dbg's flitering syntax (see
            https://ovs-dbg.readthedocs.io/en/latest/ofparse.html#filtering)

        The filtering syntax is defined as follows

        [! | not ] KEY[OPERATOR VALUE] [ && | and | || | or] ...

        Where:
            “=” checks for equality
            “<” numerical ‘less than’
            “>” numerical ‘greater than’
            “~=” mask matching (valid for fields such as IPv4, IPv6 and Ethernet)

            VALUE: The value to be compared against

            && | and: combines the filters applying logical AND
            || | or: combines the filters applying logical OR
            ! | not: applies the logical NOT to the filter

        For fields or actions that are flags (e.g: tcp or drop), the OPERATOR and VALUE can be omitted

        Examples:

        n_bytes>0 and drop
        nw_src~=192.168.1.1 or arp.tsa=192.168.1.1
        ! tcp && output.port=2
        """
        result = []
        filt = OFFilter(expr)
        for flow in self._flows:
            if filt.evaluate(flow):
                result.append(flow)
        return result



# -*- coding: utf-8 -*-

from functools import wraps
import jupyter_client
import json
import sys

client = None


class InsightsClient:
    INIT_COMMANDS = """
from functools import partial
from IPython.display import JSON
from insights.shell import Models
def kresponse(data):
    return JSON({'response': data})

def kfind_command(models, name):
    for model_name, model_cls in models.items():
        if hasattr(model_cls, 'has_commands'):
            for a_name in dir(model_cls):
                attr = getattr(model_cls, a_name)
                if hasattr(attr, 'is_cmd') and name == a_name:
                    # Evaluate the model
                    model_instance = getattr(models, model_name)
                    if model_instance:
                        return partial(attr, model_instance)
    return None

def klist_commands(models):
    res = {}
    for model_name, model_cls in models.items():
        if hasattr(model_cls, 'has_commands'):
            for a_name in dir(model_cls):
                attr = getattr(model_cls, a_name)
                if hasattr(attr, 'is_cmd'):
                    # Evaluate model to avoid returning commands of empty
                    # models
                    if getattr(models, model_name):
                        res[a_name] = attr.__doc__.strip()
    return res

def krun_command_models(models, name, *args, **kwargs):
    cmd = kfind_command(models, name)
    if not cmd:
        return None
    return cmd(*args, **kwargs)

def krun_command(name, *args, **kwargs):
    result = {}
    # FIXME: Export models' path
    for path, model in (models.items() if isinstance(models, Holder) else [(models._tmp, models)]):
        result[path] = krun_command_models(model, name, *args, **kwargs)

    return result
"""

    def __init__(self, verbose=False):
        self._client = None
        self.response = dict()
        self.output = dict()
        self.status = dict()
        self.verbose = verbose

        cf = jupyter_client.find_connection_file()
        self._client = jupyter_client.BlockingKernelClient(connection_file=cf)
        self._client.load_connection_file()
        self._run_init_commands()

    def _run_init_commands(self):
        self.__run("init", self.INIT_COMMANDS)

    def close(self):
        return self.__run("quit", "quit")

    def evaluate(self, name, subargs=""):
        command = """
        result = {{}}
        # FIXME: Export models' path
        for path, model in (models.items() if isinstance(models, Holder) else [(models._tmp, models)]):
            {name} = model.evaluate('{name}')
            if {name}:
                result[path] = {name}{subargs}

        kresponse(result)
        """.format(
            name=name, subargs="." + subargs if subargs else ""
        )

        return self.__run(name, command)

    def run_command(self, name, *args, **kwargs):
        """
        Run a command. Returns a dictionary indexed by archive name with the results
        """
        total_args = ",".join(
            filter(
                None,
                [
                    "'{}'".format(name),
                    ",".join([self._prepare_arg(arg) for arg in args]),
                    ",".join(
                        [
                            "=".join([key, self._prepare_arg(kwargs[key])])
                            for key in kwargs.keys()
                        ]
                    ),
                ],
            )
        )
        command = "krun_command({total_args})".format(name=name, total_args=total_args)
        return self._run(name, command)

    def available(self):
        """
        Returns the available models and commands in a dictionary indexed by archive
        Returns:
            dict [str: dictionary ['models': list[str], 'commands' : list[str]]]
        """
        command = """
        result = {}
        for path, model in (models.items() if isinstance(models, Holder) else [(models._tmp, models)]):
            result[path] = {
                        'models': [str(m) for m in model.keys()],
                        'commands': [" : ".join([name, help]) for name, help in klist_commands(model).items()]
                        }
        kresponse(result)
        """
        return self.__run("available", command)

    def _prepare_arg(self, arg):
        if isinstance(arg, str):
            return "'{}'".format(arg)

    def _run(self, name, cmd):
        cmd = "kresponse({})".format(cmd)
        return self.__run(name, cmd)

    def __run(self, name, cmd):
        if self.verbose:
            print("Running command: {}".format(cmd), file=sys.stderr)

        def output_hook(msg):
            msg_type = msg["header"]["msg_type"]
            content = msg["content"]
            if self.verbose:
                print(msg)
            if msg_type == "stream":
                stream = getattr(sys, content["name"])
                stream.write(content["text"])
            elif msg_type == "execute_result":
                data = content["data"].get("application/json", "").get("response")
                self.response[name] = data
            elif msg_type == "status":
                self.status[name] = content.get("execution_state", "unknown")
            elif msg_type == "error":
                print("\n".join(content["traceback"]), file=sys.stderr)

        o = self._client.execute_interactive(cmd, output_hook=output_hook)
        return self.response.get(name)



import gzip
import os
import re

from insights.core.context import FSRoots
from insights.core.spec_factory import TextFileProvider, RawFileProvider, simple_file
from insights.core.plugins import datasource


class recursive_dir(object):
    """
    Creates a datasource that recursively reads all the files in a directory

    Args:
        path (str): path of the directory to traverse
        context (ExecutionContext): the context under which the datasource
            should run.
        kind (FileProvider): One of TextFileProvider or RawFileProvider
        include (str): regular expression defining paths to include.
        ignore (str): regular expression defining paths to ignore.

    Returns:
        function: A datasource that reads all files matching the glob patterns.
    """

    def __init__(
        self,
        path,
        context=None,
        include=None,
        ignore=None,
        kind=TextFileProvider,
        deps=[],
    ):
        self.path = path
        self.context = context or FSRoots
        self.kind = kind
        self.ignore_func = re.compile(ignore).search if ignore else lambda x: False
        self.include_func = re.compile(include).search if include else lambda x: True
        self.__name__ = self.__class__.__name__
        datasource(self.context, *deps, multi_output=True, raw=kind is RawFileProvider)(
            self
        )

    def __call__(self, broker):
        ctx = broker.get(self.context)
        root = ctx.root
        path = ctx.locate_path(os.path.join(root, self.path.lstrip("/")))
        results = self._listdir(ctx, root, path)
        return results

    def _listdir(self, ctx, root, path):
        result = []
        for entry in os.listdir(path):
            entry_path = os.path.join(path, entry)
            if os.path.isdir(entry_path):
                result.extend(self._listdir(ctx, root, entry_path))
            else:
                if not self.include_func(entry_path) or self.ignore_func(entry_path):
                    continue
                result.append(
                    self.kind(entry_path[len(root) :], root=root, ds=self, ctx=ctx)
                )
        return result


class GZFileProvider(RawFileProvider):
    """
    Class used in datasources that returns the contents of a gzipped file as
    a list of lines
    """

    def load(self):
        self.loaded = True
        with gzip.open(self.path, "rt", encoding="utf-8") as f:
            return [l.rstrip("\n") for l in f]



""" Defines classes and functions that help locate commands within insights
components.

A command is nothing but an insight component (fact, combiner, parser, etc)
that has a function that can be run with arbitrary parameters
"""


class CommandMetaClass(type):
    def __init__(cls, clsname, superclasses, attributedict):
        cls.has_commands = True


def command(func):
    func.is_cmd = True
    return func



import os

from insights import parser
from insights.core.spec_factory import simple_file, simple_command

from insights_net.plugins.parsers.ovsdb import (
    OVSDBListParser,
    OVSDBParser,
    ovsdb_servers,
)

# Openstack dumps
ovn_nb_dump = simple_file(
    "sos_commands/ovn_central/podman_exec_ovn-dbs-bundle-podman-0_ovsdb-client_-f_list_dump_unix.var.run.openvswitch.ovnnb_db.sock"
)
ovn_sb_dump = simple_file(
    "sos_commands/ovn_central/podman_exec_ovn-dbs-bundle-podman-0_ovsdb-client_-f_list_dump_unix.var.run.openvswitch.ovnsb_db.sock"
)


@parser(ovn_nb_dump)
class OVNNBDump(OVSDBListParser):
    def __init__(self, *args, **kwargs):
        super(OVNNBDump, self).__init__(*args, **kwargs)
        self._name = "OVN_Northbound"


@parser(ovn_sb_dump)
class OVNSBDump(OVSDBListParser):
    def __init__(self, *args, **kwargs):
        super(OVNSBDump, self).__init__(*args, **kwargs)
        self._name = "OVN_Southbound"


# Local
ovn_nb_db = ovsdb_servers(["*.sock", "ovnnb_db.sock"], "OVN_Northbound")
ovn_sb_db = ovsdb_servers(["*.sock", "ovnsb_db.sock"], "OVN_Southbound")


@parser(ovn_nb_db)
class OVNNBLocal(OVSDBParser):
    def __init__(self, *args, **kwargs):
        super(OVNNBLocal, self).__init__(*args, **kwargs)

    def parse_content(self, data):
        self._tables = data
        self._name = "OVN_Northbound"


@parser(ovn_sb_db)
class OVNSBLocal(OVSDBParser):
    def __init__(self, *args, **kwargs):
        super(OVNSBLocal, self).__init__(*args, **kwargs)

    def parse_content(self, data):
        self._tables = data
        self._name = "OVN_Southbound"



import click
from rich.console import Console

from insights_net.main import maincli
from insights_net.commands.printing import print_archive_header


@maincli.command(name="host")
@click.pass_obj
def host(ctx):
    """
    Show basic host information
    """
    data = ctx.client.evaluate("host_info")
    if not data:
        print("Command not found")
        return

    console = Console()

    for archive, host_data in data.items():
        if not host_data:
            continue

        print_archive_header(console, archive)
        if isinstance(host_data, str):
            console.print(host_data)
        else:
            print_results(console, host_data)


def print_results(console, data):
    console.print("[bold]HostName:[/bold] {}".format(data.get("hostname")))
    rh_ver = data.get("version")
    if rh_ver and isinstance(rh_ver, dict):
        console.print("[bold]Red Hat Version:[/bold]")
        console.print("  [bold]Product:[/bold] {}".format(rh_ver.get("product")))
        console.print("  [bold]Version:[/bold] {}".format(rh_ver.get("version")))
        console.print("  [bold]Code Name:[/bold] {}".format(rh_ver.get("code_name")))

    uname = data.get("uname")
    if uname and isinstance(rh_ver, dict):
        console.print("[bold]Kernel:[/bold]")
        console.print("  [bold]Version :[/bold] {}".format(uname.get("version")))
        console.print("  [bold]Release:[/bold] {}".format(uname.get("release")))
        console.print("  [bold]Arch:[/bold] {}".format(uname.get("arch")))

    up = data.get("uptime")
    if up and isinstance(up, dict):
        console.print(
            "[bold]Uptime for {} days {} hh:[/bold]mm".format(
                up.get("updays"), up.get("uphhmm")
            )
        )

    sel = data.get("selinux")
    if sel and isinstance(sel, dict):
        console.print(
            "[bold]Selinux:[/bold] {}".format(data.get("selinux").get("selinux_status"))
        )
    console.print("")



import click
from rich.console import Console
from rich.table import Table
from rich.pretty import Pretty

from insights_net.main import maincli
from insights_net.commands.printing import print_table, print_archive_header

# The following click commands only modify the attributes of ctx.
# The command's logic will then use these commands to obtain the relevant data


@maincli.group(name="ovs")
@click.pass_obj
def ovs(ctx):
    """
    Show the OVS configuration
    """
    setattr(ctx, "cmd_table_list", "ovs_table_list")
    setattr(ctx, "cmd_table", "ovs_table")
    setattr(ctx, "cmd_find_uuid", "ovs_find_uuid")


@ovs.command(name="list")
@click.argument("table", required=False, nargs=1)
@click.option(
    "-l",
    "--list",
    "list_flag",
    is_flag=True,
    default=False,
    show_default=True,
    help="Show tables as lists of elements (useful if tables are too big)",
)
@click.pass_obj
def ovs_list(ctx, table, list_flag):
    """
    List the content of a OVS Table
    """
    return list_cmd(ctx, table, tables_as_lists=list_flag)


@ovs.command(name="get")
@click.argument("table", required=True, nargs=1)
@click.argument("uuid", required=True, nargs=1)
@click.pass_obj
def ovs_get(ctx, table, uuid):
    """
    Get an element form the OVS Database
    """
    return get_cmd(ctx, table, uuid)


@maincli.group(name="ovn")
@click.pass_obj
def ovn(ctx):
    """
    Show the OVN configuration
    """
    pass


@ovn.group(name="nb")
@click.pass_obj
def nb(ctx):
    setattr(ctx, "cmd_table_list", "ovnnb_table_list")
    setattr(ctx, "cmd_table", "ovnnb_table")
    setattr(ctx, "cmd_find_uuid", "ovnnb_find_uuid")


@nb.command(name="list")
@click.argument("table", required=False, nargs=1)
@click.option(
    "-l",
    "--list",
    "list_flag",
    is_flag=True,
    default=False,
    show_default=True,
    help="Show tables as lists of elements (useful if tables are too big)",
)
@click.pass_obj
def nb_list(ctx, table, list_flag):
    """
    List the content of a OVN NB Table
    """
    return list_cmd(ctx, table, tables_as_lists=list_flag)


@nb.command(name="get")
@click.argument("table", required=True, nargs=1)
@click.argument("uuid", required=True, nargs=1)
@click.pass_obj
def nb_get(ctx, table, uuid):
    """
    Get an element form the NB Database
    """
    return get_cmd(ctx, table, uuid)


@ovn.group(name="sb")
@click.pass_obj
def sb(ctx):
    setattr(ctx, "cmd_table_list", "ovnsb_table_list")
    setattr(ctx, "cmd_table", "ovnsb_table")
    setattr(ctx, "cmd_find_uuid", "ovnsb_find_uuid")
    pass


@sb.command(name="list")
@click.argument("table", required=False, nargs=1)
@click.option(
    "-l",
    "--list",
    "list_flag",
    is_flag=True,
    default=False,
    show_default=True,
    help="Show tables as lists of elements (useful if tables are too big)",
)
@click.pass_obj
def sb_list(ctx, table, list_flag):
    """
    List the content of a OVN SB Table
    """
    return list_cmd(ctx, table, tables_as_lists=list_flag)


@sb.command(name="get")
@click.argument("table", required=True, nargs=1)
@click.argument("uuid", required=True, nargs=1)
@click.pass_obj
def sb_get(ctx, table, uuid):
    """
    Get an element form the SB Database
    """
    return get_cmd(ctx, table, uuid)


# Commands logic


def get_cmd(ctx, table, uuid):
    data = ctx.client.run_command(ctx.cmd_find_uuid, table, uuid)
    console = Console()
    if not data:
        print("No data")
        return
    for archive, host_data in data.items():
        print_archive_header(console, "Archive: " + archive)
        if not host_data:
            print("No data")
        else:
            console.print(host_data)


def list_cmd(ctx, table, tables_as_lists=False):
    """
    List the content of a DB Tabel
    """
    console = Console()
    if not table:
        tables = ctx.client.run_command(ctx.cmd_table_list)
        if not tables:
            console.print("No data")
            return

        for archive, host_data in tables.items():
            print_archive_header(console, "Archive: " + archive)

            if not host_data:
                console.print("No data")
            elif isinstance(host_data, str):
                console.print(host_data)
            else:
                console.print("Available Tables:")
                console.print(host_data)
        return

    table_data = ctx.client.run_command(ctx.cmd_table, table)
    if not table_data:
        console.print("No data")
        return

    for archive, host_data in table_data.items():
        print_archive_header(console, "Archive: " + archive)
        if not host_data:
            console.print("No data")
            continue
        if isinstance(host_data, str):
            console.print(host_data)
        else:
            print_table(
                console,
                list(host_data.values()),
                table,
                tables_as_lists=tables_as_lists,
            )


## TODO; replace with printing.py common code
def print_results(table_data, table):
    console = Console()
    tt = Table(title=table)
    for header, value in table_data[next(iter(table_data.keys()))].items():
        if isinstance(value, dict):
            tt.add_column(header, justify="left", no_wrap=False, ratio=1, min_width=25)
        else:
            tt.add_column(header, justify="left", no_wrap=False, min_width=len(header))

    for elem in table_data.values():
        values = [Pretty(val) for val in elem.values()]
        tt.add_row(*values)

    console.print(tt)



""" OVS combiners

OVN information can be obtained from lists, dumps or live database.
Combine them all into a single combiner per database type
"""
from insights.core.plugins import combiner
from insights.parsers import SkipException

from insights_net.plugins.commands import CommandMetaClass, command
from insights_net.plugins.parsers.ovs import (
    OVSDump,
    OVSLocal,
    OVSOfctlDump,
    OVSOfctlShow,
)
from insights_net.plugins.parsers.ovsdb import OVSDBMixin


class OVSDBCommandMetaClass(CommandMetaClass):
    """
    OVSDBCommandMetaClass is a CommandMetaClass taht, used by OVSDBMixin
    instances, exposes the some functions provided by such Mixin as commands.

    It accepts one argument that is a string to be prepended to all commands
    exposed.
    """

    @classmethod
    def __prepare__(cls, name, bases, **kwargs):
        return super().__prepare__(name, bases, **kwargs)

    def __new__(cls, name, bases, namespace, **kwargs):
        return super().__new__(cls, name, bases, namespace)

    def __init__(cls, name, bases, namespace, cmd_name="", *kwargs):
        super().__init__(name, bases, namespace)
        functions_to_commands = [
            "table_list",
            "columns",
            "table",
            "row",
            "find",
            "find_uuid",
        ]

        for cmd in functions_to_commands:
            full_name = "{db}_{cmd}".format(db=cmd_name, cmd=cmd)
            orig_func = getattr(cls, cmd)
            doc = orig_func.__doc__

            setattr(cls, full_name, cls._create_wrapper(orig_func, full_name, doc))

    @classmethod
    def _create_wrapper(cls, orig_func, name, doc):
        def func(self, *args, **kwargs):
            return orig_func(self, *args, **kwargs)

        func.__name__ = name
        func.__doc__ = doc
        return command(func)


class OVSDBCombiner(OVSDBMixin):
    """
    OVSDBCombiner is a base class for OVSDBMixin combiners that combine
    dump, local and ocp OVSDB Parsers
    """

    def __init__(self, dump, local, ocp):
        if local:
            # Prefer local
            if isinstance(local, list):
                local = local[0]
            self._tables = local.tables
            self._name = local.name
        elif dump:
            if isinstance(dump, list):
                dump = dump[0]
            self._tables = dump.tables
            self._name = dump.name
        elif ocp:
            if isinstance(ocp, list):
                ocp = ocp[0]
            self._tables = ocp.tables
            self._name = ocp.name
        else:
            raise Exception("No OVSDB data available")


@combiner([OVSDump, OVSLocal])
class OVS(OVSDBCombiner, metaclass=OVSDBCommandMetaClass, cmd_name="ovs"):
    def __init__(self, dump, local):
        super(OVS, self).__init__(dump, local, None)


# Ofproto


@combiner(OVSOfctlShow, OVSOfctlDump)
class OVSOfproto(metaclass=CommandMetaClass):
    """OVSOfproto groups all the Ofproto information in a single object
    and exposes some commands
    """

    def __init__(self, shows, dumps):
        self.bridges = {}
        if len(shows) != len(dumps):
            raise SkipException("Uncomplete data")

        for show in shows:
            if not self.bridges.get(show.bridge_name):
                self.bridges[show.bridge_name] = {}

            self.bridges[show.bridge_name]["show"] = show

        for dump in dumps:
            if not self.bridges.get(dump.bridge_name):
                raise SkipException("Uncomplete data")

            self.bridges[dump.bridge_name]["dump"] = dump

    @command
    def ofproto_summary(self):
        """Return a summary of the ofproto information"""
        return {
            name: {"ports": bridge["show"].ports, "flows": bridge["dump"].flow_list.len}
            for name, bridge in self.bridges
        }

    @command
    def find_flows(self, expr):
        """Find flows that match a certain expression (see ovs-dbg filtering)"""
        return {
            name: [str(f) for f in bridge["dump"].flow_list.find(expr)]
            for name, bridge in self.bridges
        }

    @command
    def show(self):
        """Return the openflow information ('ofctl show')"""
        return {name: bridge["show"] for name, bridge in self.bridges}

    @command
    def flows(self):
        """Return all the openflow flows"""
        return {
            name: [str(f) for f in bridge["dump"].flow_list.flows]
            for name, bridge in self.bridges
        }



from insights import fact

from insights.combiners.hostname import Hostname
from insights.combiners.selinux import SELinux
from insights.parsers.redhat_release import RedhatRelease
from insights.parsers.uname import Uname
from insights.parsers.uptime import Uptime


@fact(optional=[Hostname, RedhatRelease, SELinux, Uname, Uptime])
def host_info(hostname, release, selinux, uname, up):
    """
    Returns a sumary of the host information
    """
    return {
        "hostname": hostname.fqdn if hostname else "Unavailable",
        "version": release.parsed if release else "Unavailable",
        "selinux": selinux.sestatus.data if selinux else "Unavailable",
        "uname": {
            "version": uname.data.get("version") if uname else "Unavailable",
            "release": uname.data.get("release") if uname else "Unavailable",
            "arch": uname.data.get("arch") if uname else "Unavailable",
        },
        "uptime": {
            "updays": up.updays if up else "Unavailable",
            "uphhmm": up.uphhmm if up else "Unavailable",
            "loadavg": up.loadavg if up else "Unavailable",
        },
    }



import click
import yaml
from rich.console import Console
from rich.text import Text
from rich.panel import Panel
from rich.emoji import Emoji
from rich.style import Style
from ovs_dbg.ofp import OFPFlow
from ovs_dbg.filter import OFFilter

from insights_net.main import maincli

from insights_net.commands.printing import (
    print_table,
    print_section_header,
    print_archive_header,
)


@maincli.command(name="find-ip")
@click.argument("address", required=True, nargs=1)
@click.option(
    "-l",
    "--list",
    "list_flag",
    is_flag=True,
    default=False,
    show_default=True,
    help="Show tables as lists of elements (useful if tables are too big)",
)
@click.pass_obj
def find_ip(ctx, address, list_flag):
    """
    Get all the available information regarding an IP(v4/6) address
    """
    data = ctx.client.run_command("find_ip", ip=address)
    if not data:
        print("Command not found")
        return

    console = Console()

    for archive, host_data in data.items():
        if not host_data:
            continue

        print_archive_header(console, archive)

        if isinstance(host_data, str):
            console.print(host_data)
        else:
            print_results(console, host_data, list_flag)
        print("")


def print_results(console, data, tables_as_lists=False):
    host_matches = data.get("ip_addr")
    if host_matches:
        for netns, match in host_matches.items():
            if len(match) > 0:
                print_section_header(
                    console, "Network Interface Matches (Namespace: {})".format(netns)
                )
                for iface in match:
                    console.print("  - Name: {}".format(iface.get("name")))
                    console.print("    Type: {}".format(iface.get("type")))
                    console.print("    Addresses:")
                    for addr in iface.get("addr"):
                        console.print(
                            "     - {}/{}".format(addr.get("addr"), addr.get("mask"))
                        )
                    console.print("    MAC: {}".format(iface.get("mac")))
                    console.print("    MTU: {}".format(iface.get("mtu")))
                    console.print("    State: {}".format(iface.get("state")))
                    console.print("    QDisc: {}".format(iface.get("qdisc")))
                    console.print("")
                console.print("")
        console.print("")

    neigh_matches = data.get("ip_neigh")
    if neigh_matches:
        print_section_header(console, "Neighbor Matches")
        for neigh in neigh_matches:
            console.print("  Address: {}".format(neigh.get("addr")))
            console.print("  Device: {}".format(neigh.get("dev")))
            console.print("  LLAdr: {}".format(neigh.get("lladdr")))
            console.print("  Reachibility: {}".format(neigh.get("nud")))
            console.print("")
        console.print("")

    route_matches = data.get("ip_route")
    if route_matches:
        for netns, match in route_matches.items():
            print_section_header(console, "Route Matches (Namespace: {})".format(netns))
            for route in match:
                console.print("{}: ".format(route.get("match")))
                for entry in route.get("routes"):
                    console.print(
                        "    - [bold]Prefix:[/bold] {}".format(entry.get("prefix"))
                    )
                    console.print("      [bold]Via:[/bold] {}".format(entry.get("via")))
                    console.print("      [bold]Dev:[/bold] {}".format(entry.get("dev")))
                    console.print(
                        "      [bold]Table:[/bold] {}".format(entry.get("table"))
                    )
                    console.print(
                        "      [bold]Metric:[/bold] {}".format(entry.get("metric"))
                    )
                    console.print(
                        "      [bold]Pref:[/bold] {}".format(entry.get("pref"))
                    )
                    console.print("")
                console.print("")

    hosts_matches = data.get("hosts")
    if host_matches and len(hosts_matches) > 0:
        print_section_header(console, "Hosts Matches")
        for host in hosts_matches:
            console.print("    - [bold] Hostname:[/bold] {}".format(host))
        console.print("")

    ipt_matches = data.get("iptables")
    if ipt_matches:
        for config, matches in ipt_matches.items():
            if len(matches) > 0:
                print_section_header(console, "{} Matches".format(config))
                for match in matches:
                    rule = match.get("rule")
                    console.print(
                        "    - [bold]Chain:[/bold] {}".format(match.get("chain"))
                    )
                    console.print(
                        "      [bold]Table:[/bold] {}".format(rule.get("table"))
                    )
                    console.print(
                        "      [bold]Rule:[/bold] {}".format(rule.get("rule"))
                    )
                console.print("")
        console.print("")

    ns_matches = data.get("netstat")
    if ns_matches:
        for where in ["Local", "Foreign"]:
            matches = ns_matches.get(where)
            if matches:
                print_section_header(
                    console, "Netstat {} Address Matches".format(where)
                )
                print_table(console, matches, "Netstat " + where, tables_as_lists)
                console.print("")

    ofctl_matches = data.get("ofctl")
    if ofctl_matches:
        for bridge in ofctl_matches.keys():
            flows = ofctl_matches.get(bridge)
            if flows:
                parsed_flows = [OFPFlow.from_string(f) for f in flows]
                print_section_header(
                    console, "Ofproto flow Matches on bridge {}".format(bridge)
                )
                console.print("")

                drop_filter = OFFilter("drop")

                drops = list(filter(drop_filter.evaluate, parsed_flows))

                if drops:
                    console.print("[bold] DROPS:[/bold]")
                    print_ofproto_flows(console, drops)

                console.print("[bold] ALL FLOWS:[/bold]")
                print_ofproto_flows(console, parsed_flows)
        console.print("")

    ovs_matches = data.get("ovs")
    if ovs_matches:
        print_section_header(console, "OpenvSwitch Configuration Matches")
        for table, rows in ovs_matches.items():
            console.print("   * Table {}".format(table))
            print_table(console, rows, table, tables_as_lists)
            console.print("")
        console.print("")

    nb_matches = data.get("nb")
    if nb_matches:
        print_section_header(console, "OVN North Bound Matches")
        for table, rows in nb_matches.items():
            console.print("   * Table {}".format(table))
            print_table(console, rows, table, tables_as_lists)
            console.print("")
        console.print("")

    sb_matches = data.get("sb")
    if sb_matches:
        print_section_header(console, "OVN South Bound Matches")
        for table, rows in sb_matches.items():
            console.print("   * Table {}".format(table))
            print_table(console, rows, table, tables_as_lists)
            console.print("")
        console.print("")

    pods_matches = data.get("pods")
    if pods_matches:
        print_section_header(console, "OCP Pod Matches")
        for pod in pods_matches:
            console.print(
                "   * Pod Name: {}  Namespace {} matches in field {}: {}".format(
                    pod.get("name"),
                    pod.get("namespace"),
                    pod.get("field"),
                    pod.get("match"),
                )
            )
            console.print("   * Pod full config:")
            console.print(yaml.dump(pod.get("full")))
            console.print("")
        console.print("")

    services_matches = data.get("services")
    if services_matches:
        print_section_header(console, "OCP Services Matches")
        for service in services_matches:
            console.print(
                "   * Service Name: {}  Namespace {} matches in field {}: {}".format(
                    service.get("name"),
                    service.get("namespace"),
                    service.get("field"),
                    service.get("match"),
                )
            )
            console.print("   * service full config:")
            console.print(yaml.dump(service.get("full")))
            console.print("")
        console.print("")

    ocpnet_matches = data.get("ocp_net")
    if ocpnet_matches:
        print_section_header(console, "OCP Network Configuration Matches")
        for conf in ocpnet_matches:
            console.print(
                "   * Network Name: {} matches in field {}: {}".format(
                    conf.get("name"), conf.get("field"), conf.get("match")
                )
            )
            console.print("   * Network full config:")
            console.print(yaml.dump(conf.get("full")))
            console.print("")
        console.print("")


def print_ofproto_flows(console, flows):
    """
    Args:
        console (rich.Console) console to print
        flows (list[OFPFlow]): list of flows to print
    """
    for table in set([flow.info.get("table") for flow in flows]):
        console.print("   * Table {}".format(table))
        for flow in filter(lambda f: f.info.get("table") == table, flows):
            console.print(Text("      {} ".format(str(flow)), style=Style()))
    console.print("")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""The setup script."""

from setuptools import setup, find_packages

with open("README.md") as readme_file:
    readme = readme_file.read()

requirements = [
    "ipython",
    "coverage",
    "ipykernel",
    "click",
    "colorama",
    "rich",
    "ovsdbapp",
    "insights-core",
    "ovs-dbg>=0.0.10",
]

setup_requirements = [
    "pytest-runner",
]

test_requirements = [
    "pytest",
    "pylint",
]

setup(
    author="Adrián Moreno",
    author_email="amorenoz@redhat.com",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
    description="Network analysis tools based on insights-core",
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    name="insights-net",
    packages=find_packages(include=["insights_net*"]),
    setup_requires=setup_requirements,
    scripts=["bin/insights-net"],
    test_suite="tests",
    tests_require=test_requirements,
    version="0.0.2",
    zip_safe=False,
)



import re
import sys
from abc import ABC, abstractmethod


class DecodeError(Exception):
    pass


class FieldDecoder(ABC):
    @abstractmethod
    def regexp_str(self):
        pass

    @classmethod
    def decode_field(self, string):
        pass


DEFAULT_REGEXP = r"[\w]+"
MAC_REGEXP = r"[\w:]+"
IP_REGEXP = r"[\d.]+"
DEC_REGEXP = r"\d+"
HEX_REGEXP = r"[x0-9A-Fa-f]+"
DATA_REGEXP = r"[\.0-9A-Fa-f]+"


class StringFieldDecoder(FieldDecoder):
    def __init__(self, regexp):
        self._regex = regexp

    def regexp_str(self):
        return self._regex

    @classmethod
    def decode_field(cls, string):
        return string


IPFieldDecoder = StringFieldDecoder(IP_REGEXP)
MACFieldDecoder = StringFieldDecoder(MAC_REGEXP)
SimpleStringFieldDecoder = StringFieldDecoder(DEFAULT_REGEXP)


class SubFieldDecoder(FieldDecoder):
    SUBFIELD_REGEXP = r"\w+\[(?:\d+)?(?:..)?(?:\d+)?\]"
    SUBFIELD_SEL_REGEXP = r"(\w+)\[(\d+)?(?:..)?(\d+)?\]"
    sel_regexp = re.compile(SUBFIELD_SEL_REGEXP)

    @classmethod
    def regexp_str(cls):
        return cls.SUBFIELD_REGEXP

    @classmethod
    def decode_field(cls, string):
        match = cls.sel_regexp.match(string)
        if match is None:
            raise DecodeError("Failed to decode subfield %s" % string)

        sf = {
            "field": match.group(1),
        }
        if start := match.group(2):
            sf["start"] = int(start)
        if end := match.group(3):
            sf["end"] = int(end)
        return sf


class DataFieldDecoder(FieldDecoder):
    DATA_REGEXP = r"[\.0-9A-Fa-f]+"

    @classmethod
    def regexp_str(cls):
        return DATA_REGEXP

    @classmethod
    def decode_field(cls, string):
        return bytes.fromhex(string.replace(".", ""))


a = DataFieldDecoder()


class IntFieldDecoder(FieldDecoder):
    def __init__(self, regexp):
        self._regexp = regexp

    def regexp_str(self):
        return self._regexp

    @classmethod
    def decode_field(cls, string):
        return int(string, 0)


DecFieldDecoder = IntFieldDecoder(DEC_REGEXP)
HexFieldDecoder = IntFieldDecoder(HEX_REGEXP)


class ActionDecoder(ABC):
    @abstractmethod
    def regexp(self):
        """
        Must return the regexp object
        """
        pass

    @abstractmethod
    def decode_action(self, actions, match):
        """
        Populates the action object based on match
        """
        pass


class SingleKeyValueDecoder(ActionDecoder):
    def __init__(self, keyword, regexp, field_decoder=SimpleStringFieldDecoder):
        self._keyword = keyword
        self._regex = re.compile(
            regexp.format(self._keyword, field_decoder.regexp_str())
        )
        self._field_decoder = field_decoder

    def regexp(self):
        return self._regex

    def decode_action(self, actions, match):
        value = self._field_decoder.decode_field(match.group(1))

        actions.append({"action": self._keyword, "params": value})


class ColonDecoder(SingleKeyValueDecoder):
    """
    Decodes actions such as 'keyword:argument'
    """

    REGEXP = r"{}:({})"

    def __init__(self, keyword, field_decoder=SimpleStringFieldDecoder):
        super(ColonDecoder, self).__init__(keyword, self.REGEXP, field_decoder)


class ParenthesisDecoder(SingleKeyValueDecoder):
    """
    Decodes actions such as 'keyword(argument)'
    """

    REGEXP = r"{}\(({})\)"

    def __init__(self, keyword, field_decoder=SimpleStringFieldDecoder):
        super(ParenthesisDecoder, self).__init__(keyword, self.REGEXP, field_decoder)


class KeywordDecoder(ActionDecoder):
    def __init__(self, keyword):
        self._keyword = keyword

    def regexp(self):
        return re.compile(self._keyword)

    def decode_action(self, actions, match):
        actions.append({"action": self._keyword})
        return True


# TODO: Convert into parenthesyslist instance
# class CTDecoder(ActionDecoder):
#    @classmethod
#    def regexp(cls):
#        reg = r'ct\((commit)?,?(force)?,?(table=(\d+))?,?(zone=(\w+)\[(\d+)?(..)?(\d+)?\])?,?(zone=(\d+))?,?(nat)?,?({})?,?(exec\((.*)\))?\)'.format(
#            NatDecoder().regexp_str())
#        return re.compile(reg)
#
#    @classmethod
#    def decode_action(cls, actions, match):
#        params = {}
#
#        # (commit)?
#        if match.group(1):
#            params["commit"] = True
#
#        # (force)?
#        if match.group(2):
#            params["force"] = True
#
#        # (table=(\d+))
#        if match.group(3):
#            params["table"] = int(match.group(4))
#
#        # (zone=(\w+)\[(\d+)?(..)?(\d+)?\])?,?(zone=(\d+))?
#        if match.group(5):
#            params["zone"] = {
#                "source": match.group(6),
#            }
#            if start := match.group(7):
#                params["zone"]["start"] = start
#            if end := match.group(9):
#                params["zone"]["end"] = end
#
#        # (zone=(\d+))?
#        if match.group(10):
#            params["zone_imm"] = match.group(11)
#
#        # (nat)?
#        if match.group(12):
#            params["nat"] = True
#
#        # (nat(...))?
#        if match.group(13):
#            params["nat"] = NatDecoder().decode_field(match.group(13))
#
#        # ?(exec\((.*)\))??
#        if match.group(14):
#            params["exec"] = decode_action_line(match.group(15))
#
#        actions.append({
#            "action": "ct",
#            "params": params
#        })
#


class NatDecoder(ActionDecoder, FieldDecoder):
    """
    Nat can, itself, be an action or a field in the "ct" action
    """

    REGEXP_GROUP = r"nat\((\w+)?(=[\w\d\.:\[\]-]*)?,?(\w+)?,?(\w+)?,?(\w+)?\)"
    REGEXP_FIELD = r"nat\((?:\w+)?(?:=[\w\d\.:\[\]-]*)?,?(?:\w+)?,?(?:\w+)?,?(?:\w+)?\)"
    _field_regex = re.compile(REGEXP_GROUP)
    _port_regex = re.compile("(?:[\w.:\]\[]+)(?:-[\w.:\]\[]+)?(?::(\d+-\d+))")
    # Action Decoder Implementatio'n
    @classmethod
    def regexp(cls):
        return re.compile(cls.REGEXP_GROUP)

    @classmethod
    def decode_action(cls, actions, match):
        params = cls._decode_match(match)
        actions.append({"action": "nat", "params": params})

    # Field Decoder Implementation
    @classmethod
    def decode_field(cls, string):
        match = cls._field_regex.match(string)
        if not match:
            raise DecodeError("Could not decode nat field: {}".format(string))
        return cls._decode_match(match)

    @classmethod
    def regexp_str(cls):
        return cls.REGEXP_FIELD

    @classmethod
    def _decode_match(cls, match):
        params = {}
        if match.group(1) is None:
            raise DecodeError("wrong nat format")

        nat_type = match.group(1)
        params["type"] = nat_type

        if match.group(2):
            # get rid of =
            addr_str = match.group(2)[1:]

            # get ports
            # We cannot simply split by ':' because of IPV6 addresses
            # Extract the ports by regexp
            port_match = cls._port_regex.match(addr_str)
            if port_match is not None:
                ports = port_match.group(1)
                port_range = ports.split("-")
                params["port_min"] = port_range[0]
                if len(port_range) == 2:
                    params["port_max"] = port_range[1]

                # Remove the ports from the address string, including ":"
                addr_str = addr_str[: len(addr_str) - len(ports) - 1]

            # get IP range
            ip_range = addr_str.split("-")
            if len(ip_range) == 2:
                params["addr_min"] = ip_range[0].strip("[]")
                params["addr_max"] = ip_range[1].strip("[]")
            else:
                params["addr"] = addr_str.strip("[]")

        for flag in ["persistent", "hash", "random"]:
            if flag in [match.group(3), match.group(4), match.group(5)]:
                params[flag] = True

        return params


class ResubmitDecoder(ActionDecoder):
    @classmethod
    def regexp(cls):
        reg = r"(commit,)?(force,)?(table=(\d+))?"
        return re.compile(r"resubmit\((\w+)?,(\d+)(,ct)?\)")

    @classmethod
    def decode_action(cls, actions, match):
        """
        Decodes the resubmit action
        """
        params = {}
        if not match.group(1) and not match.group(2):
            raise DecodeError("expected inport or table")

        if match.group(1):
            params["inport"] = match.group(1)
        if match.group(2):
            params["table"] = int(match.group(2))

        params["ct"] = match.group(3) is not None

        actions.append({"action": "resubmit", "params": params})


##class PushPopDecoder(ActionDecoder):
##    @classmethod
##    def regexp(cls):
##        return re.compile(r'(push|pop):(\w+)\[(\d+)?(...)?(\d+)?\]')
##
##    @classmethod
##    def decode_field(cls, actions, match):
##        """
##        Decodes the resubmit action
##        """
##        action = match.group(1)
##        params = {}
##        if match.group(2):
##            params["destination"] = match.group(2)
##        else:
##            raise DecodeError("no destination in PushPopDecoder")
##
##        if match.group(3):
##            params["start"] = match.group(3)
##
##        if match.group(5):
##            params["end"] = match.group(5)
##
##        actions.append({
##            "action": action,
##            "params": params
##        })


class LoadDecoder(ActionDecoder):
    @classmethod
    def regexp(cls):
        return re.compile(r"load:(\w+)->({})".format(SubFieldDecoder.regexp_str()))

    @classmethod
    def decode_action(cls, actions, match):
        params = {}
        if match.group(1):
            params["value"] = int(match.group(1), 0)
        if match.group(2):
            params["destination"] = SubFieldDecoder.decode_field(match.group(2))

        actions.append({"action": "load", "params": params})
        return True


class SubActionFieldDecoder(FieldDecoder):
    """
    Decodes an action that has another action string in it, e.g:
        clone(action_str)
    """

    REGEXP_FIELD = r".*"

    @classmethod
    def regexp_str(cls):
        return cls.REGEXP_FIELD

    @classmethod
    def decode_field(cls, string):
        return decode_action_line(string)


class MoveDecoder(ActionDecoder):
    @classmethod
    def regexp(cls):
        sf_regex = SubFieldDecoder.regexp_str()
        return re.compile("move:({})->({})".format(sf_regex, sf_regex))

    @classmethod
    def decode_action(cls, actions, match):
        params = {}
        if match.group(1):
            params["source"] = SubFieldDecoder.decode_field(match.group(1))
        else:
            raise DecodeError("no source registry in move action")

        if match.group(2):
            params["destination"] = SubFieldDecoder.decode_field(match.group(2))
        else:
            raise DecodeError("no destination registry in move action")

        actions.append({"action": "move", "params": params})
        return True


class ParenthesisFieldDecoder(FieldDecoder):
    def __init__(self, keyword, field_decoder):
        self._keyword = keyword
        self._field_decoder = field_decoder
        self._regex = r"{}\({}\)".format(keyword, field_decoder.regexp_str())

    def regexp_str(self):
        return self._regex

    def decode_field(self, string):
        # remove "keyword(" and send to field_decoder
        substr = string[len(self._keyword) :].strip("()")
        value = self._field_decoder.decode_field(substr)
        return value


class EqualKeyValFieldDecoder(FieldDecoder):
    def __init__(self, keyword, field_decoder):
        self._keyword = keyword
        self._field_decoder = field_decoder
        self._regex = r"{}={}".format(keyword, field_decoder.regexp_str())

    def regexp_str(self):
        return self._regex

    def decode_field(self, string):
        # remove "keyword=" and send to field_decoder
        substr = string[len(self._keyword) + 1 :]
        value = self._field_decoder.decode_field(substr)
        return value


class FlagFieldDecoder(FieldDecoder):
    def __init__(self, keyword):
        self._keyword = keyword

    def regexp_str(self):
        return self._keyword

    def decode_field(self, string):
        return string == self._keyword


class ParenthesisListDecoder(ActionDecoder):
    """
    Decodes comma separated list of fields. E.g:
        keyword:(f1,f2,f3)
    Args:
        keyword: the keyword string of the action
        fields: list of tuples (field_name, field_decoder)

    """

    def __init__(self, keyword, fields):
        # just to be sure
        self._fields = fields
        field_regexp = ",?".join(
            [r"({})?".format(field[1].regexp_str()) for field in fields]
        )
        regexp_str = r"{}\({}\)".format(keyword, field_regexp)
        self._regexp = re.compile(regexp_str)
        self._fields = fields
        self._keyword = keyword

    def regexp(self):
        return self._regexp

    def decode_action(self, actions, match):
        params = {}
        for i in range(1, len(self._fields) + 1):
            field = self._fields[i - 1]
            field_name = field[0]
            field_decoder = field[1]
            if match.group(i):
                value = field_decoder.decode_field(match.group(i))
                params[field_name] = value

        actions.append({"action": self._keyword, "params": params})


ControllerDecoder = ParenthesisListDecoder(
    "controller",
    [
        ("reason", EqualKeyValFieldDecoder("reason", SimpleStringFieldDecoder)),
        ("max_len", EqualKeyValFieldDecoder("max_len", DecFieldDecoder)),
        ("id", EqualKeyValFieldDecoder("id", SimpleStringFieldDecoder)),
        ("userdata", EqualKeyValFieldDecoder("userdata", DataFieldDecoder())),
        ("pause", FlagFieldDecoder("pause")),
        ("meter_id", EqualKeyValFieldDecoder("meter_id", SimpleStringFieldDecoder)),
    ],
)

# e.g: multipath(eth_src,50,modulo_n,1,0,NXM_NX_REG0[])
MultipathDecoder = ParenthesisListDecoder(
    "multipath",
    [
        ("fields", SimpleStringFieldDecoder),
        ("basis", DecFieldDecoder),
        ("algorithm", SimpleStringFieldDecoder),
        ("max_link", DecFieldDecoder),
        ("arg", DecFieldDecoder),
        ("subfield", SubFieldDecoder),
    ],
)

CTDecoder = ParenthesisListDecoder(
    "ct",
    [
        ("commit", FlagFieldDecoder("commit")),
        ("force", FlagFieldDecoder("force")),
        ("table", EqualKeyValFieldDecoder("table", DecFieldDecoder)),
        ("zone", EqualKeyValFieldDecoder("zone", SubFieldDecoder)),
        ("zone_imm", EqualKeyValFieldDecoder("zone", DecFieldDecoder)),
        ("zone_imm", EqualKeyValFieldDecoder("zone", DecFieldDecoder)),
        ("nat", FlagFieldDecoder("nat")),
        ("nat", NatDecoder()),
        ("exec", ParenthesisFieldDecoder("exec", SubActionFieldDecoder())),
    ],
)


class BundleMembersFieldDecoder:
    REGEXP = "members:[\d,]+"

    @classmethod
    def regexp_str(cls):
        return cls.REGEXP

    @classmethod
    def decode_field(self, string):
        members = string.split(":")[1]
        return [int(i) for i in members.split(",")]


# e.g: bundle(eth_src,0,hrw,ofport,members:4,8)
BundleDecoder = ParenthesisListDecoder(
    "bundle",
    [
        ("fields", SimpleStringFieldDecoder),
        ("basis", DecFieldDecoder),
        ("algorithm", SimpleStringFieldDecoder),
        ("ofport", SimpleStringFieldDecoder),
        ("subfield", SubFieldDecoder),
        ("members", BundleMembersFieldDecoder()),
    ],
)

# class ControllerDecoder(ActionDecoder):
#    @classmethod
#    def regexp(cls):
#        return re.compile(
#            r'controller\((reason=(\w+))?,?(max_len=(\d+))?,?(id=(\w+))?,?(userdata=([\w\.]+))?,?(pause)?,?(meter_id=(\w+))?\)')
#
#    @classmethod
#    def decode_field(cls, actions, match):
#        params = {}
#        if match.group(1):
#            params["reason"] = match.group(2)
#
#        if match.group(3):
#            params["max_len"] = int(match.group(4), 0)
#
#        if match.group(5):
#            params["id"] = int(match.group(6), 0)
#
#        if match.group(7):
#            params["userdata"] = bytes.fromhex(match.group(8).replace('.', ''))
#
#        if match.group(9):
#            params["pause"] = True
#
#        if match.group(10):
#            params["meter_id"] = int(match.group(11), 0)
#
#        actions.append({
#             "action": "controller",
#             "params": params
#        })
#


class ConjunctionDecoder:
    @classmethod
    def regexp(cls):
        return re.compile(r"conjunction\((\d+),(\d+)/(\d+)\)")

    @classmethod
    def decode_action(cls, actions, match):
        params = {}
        params["id"] = match.group(1)
        params["clause"] = match.group(2)
        params["n_clauses"] = match.group(3)

        actions.append({"action": "conjunction", "params": params})


class EnqueueDecoder:
    @classmethod
    def regexp(cls):
        return re.compile(r"enqueue:(\d+):(\d+)")

    @classmethod
    def decode_action(cls, actions, match):
        params = {}
        params["port"] = match.group(1)
        params["queue"] = match.group(2)

        actions.append({"action": "conjunction", "params": params})


action_decoders = {
    "drop": KeywordDecoder("drop"),
    "exit": KeywordDecoder("exit"),
    "ct_clear": KeywordDecoder("ct_clear"),
    "dec_ttl": KeywordDecoder("dec_ttl"),
    "strip_vlan": KeywordDecoder("strip_vlan"),
    "pop_queue": KeywordDecoder("pop_queue"),
    "dec_mpls_ttl": KeywordDecoder("dec_mpls_ttl"),
    "dec_nsh_ttl": KeywordDecoder("dec_nsh_ttl"),
    "mod_dl_src": ColonDecoder("mod_dl_src", MACFieldDecoder),
    "mod_dl_dst": ColonDecoder("mod_dl_dst", MACFieldDecoder),
    "mod_nw_src": ColonDecoder("mod_nw_src", IPFieldDecoder),
    "mod_nw_dst": ColonDecoder("mod_nw_dst", IPFieldDecoder),
    "mod_tp_src": ColonDecoder("mod_tp_src", DecFieldDecoder),
    "mod_tp_dst": ColonDecoder("mod_tp_dst", DecFieldDecoder),
    "mod_vlan_vid": ColonDecoder("mod_vlan_vid", DecFieldDecoder),
    "mod_vlan_pcp": ColonDecoder("mod_vlan_pcp", DecFieldDecoder),
    "mod_nw_tos": ColonDecoder("mod_nw_tos", DecFieldDecoder),
    "mod_nw_ecn": ColonDecoder("mod_nw_ecn", DecFieldDecoder),
    "output": [
        ColonDecoder("output", DecFieldDecoder),
        ColonDecoder("output", SubFieldDecoder),
    ],
    "set_tunnel": ColonDecoder("set_tunnel", HexFieldDecoder),
    "set_tunnel64": ColonDecoder("set_tunnel64", DecFieldDecoder),
    "set_queue": ColonDecoder("set_queue", DecFieldDecoder),
    "note": ColonDecoder("note", DataFieldDecoder()),
    "group": ColonDecoder("group", DecFieldDecoder),
    "CONTROLLER": KeywordDecoder("CONTROLLER"),
    "NORMAL": KeywordDecoder("NORMAL"),
    "LOCAL": KeywordDecoder("LOCAL"),
    "push": ColonDecoder("push", SubFieldDecoder),
    "pop": ColonDecoder("pop", SubFieldDecoder),
    "load": LoadDecoder(),
    "move": MoveDecoder(),
    "resubmit": ResubmitDecoder,
    "ct": CTDecoder,
    "controller": ControllerDecoder,
    "multipath": MultipathDecoder,
    "bundle": BundleDecoder,
    "conjunction": ConjunctionDecoder(),
    "enqueue": EnqueueDecoder(),
    "set_mpls_ttl": ParenthesisDecoder("set_mpls_ttl", DecFieldDecoder),
    "set_mpls_tc": ParenthesisDecoder("set_mpls_tc", DecFieldDecoder),
    "set_mpls_label": ParenthesisDecoder("set_mpls_label", DecFieldDecoder),
    "push_mpls": ColonDecoder("push_mpls", HexFieldDecoder),
    "pop_mpls": ColonDecoder("pop_mpls", HexFieldDecoder),
    "nat": NatDecoder(),
    "clone": ParenthesisDecoder("clone", SubActionFieldDecoder),
}

# TODO: The following actions are not decodable yet
# bundle_load
# learn
# fin_timeout
# dec_ttl
# sample
# actions=output(port=1,max_len=100)
# check_pkt_larger
# set_field
# delete_field
# encap
# decap
# output_trunc


def decode_action_line(line):
    actions = []
    action_regex = re.compile(r"(\w+)")
    while line != "":
        some_match = False
        # Match the action name
        act_name = action_regex.match(line)
        if act_name is None:
            raise DecodeError("Failed to get action name: {}".format(line))
        name = act_name.group(1)

        decoders = []
        decoder = action_decoders.get(name, None)
        if not decoder:
            raise DecodeError(
                "Could not find a decoder for action name {}".format(name)
            )
        if isinstance(decoder, list):
            decoders.extend(decoder)
        else:
            decoders.append(decoder)

        for decoder in decoders:
            got_match = False
            regexp = decoder.regexp()
            match = regexp.match(line)
            if match:
                got_match = True
                decoder.decode_action(actions, match)
                some_match = True
                line = line[match.end() :].strip()
                if len(line) > 0 and line[0] == ",":
                    line = line[1:]
                break
        if not got_match:
            raise DecodeError(
                "All decoders for action action {} failed to match. Line: {}".format(
                    name, line
                )
            )

    return actions



""" OVN combiners

OVN information can be obtained from lists, dumps or live database.
Combine them all into a single combiner per database type
"""

from insights.core.plugins import combiner

from insights_net.plugins.parsers.ovn import (
    OVNNBDump,
    OVNSBDump,
    OVNNBLocal,
    OVNSBLocal,
)
from insights_net.plugins.parsers.ocp_net import OCPNB, OCPSB
from insights_net.plugins.combiners.ovs import OVSDBCommandMetaClass, OVSDBCombiner


@combiner([OVNNBDump, OVNNBLocal, OCPNB])
class OVNNB(OVSDBCombiner, metaclass=OVSDBCommandMetaClass, cmd_name="ovnnb"):
    def __init__(self, dump, local, ocp):
        super(OVNNB, self).__init__(dump, local, ocp)


@combiner([OVNSBDump, OVNSBLocal, OCPSB])
class OVNSB(OVSDBCombiner, metaclass=OVSDBCommandMetaClass, cmd_name="ovnsb"):
    def __init__(self, dump, local, ocp):
        super(OVNSB, self).__init__(dump, local, ocp)



"""
Extensions to insights/parsers/ip.py
TODO: send PR to insights-core
"""
import re

from insights import parser
from insights.parsers import SkipException
from insights.parsers.ip import IpAddr, RouteDevices
from insights.core.context import SosArchiveContext
from insights.core.spec_factory import glob_file

ip_netns_ipaddr = glob_file(
    "/sos_commands/networking/ip_netns_exec_*_ip_address_show",
    context=SosArchiveContext,
)

ip_netns_iproute = glob_file(
    "/sos_commands/networking/ip_netns_exec_*_ip_route_show_table_all",
    context=SosArchiveContext,
)


@parser(ip_netns_ipaddr)
class NetNsIpAddr(IpAddr):
    """
    Extends IpAddr to also parse all namespaces
    """

    FILE_RE_STR = "ip_netns_exec_([\w_-]+)_ip_address_show"
    file_re = re.compile(FILE_RE_STR)

    def __init__(self, *args, **kwargs):
        super(NetNsIpAddr, self).__init__(*args, **kwargs)

    @property
    def netns(self):
        return self._netns

    def parse_content(self, content):
        match = self.file_re.match(self.file_name)
        if not match or not match.group(1):
            raise SkipException("Wrong file name")

        self._netns = match.group(1)

        return IpAddr.parse_content(self, content)


@parser(ip_netns_iproute)
class NetNsIpRoute(RouteDevices):
    """
    Extends RouteDevices to also parse all namespaces
    """

    FILE_RE_STR = "ip_netns_exec_([\w_-]+)_ip_route_show_table_all"
    file_re = re.compile(FILE_RE_STR)

    def __init__(self, *args, **kwargs):
        super(NetNsIpRoute, self).__init__(*args, **kwargs)

    @property
    def netns(self):
        return self._netns

    def parse_content(self, content):
        match = self.file_re.match(self.file_name)
        if not match or not match.group(1):
            raise SkipException("Wrong file name")

        self._netns = match.group(1)

        return RouteDevices.parse_content(self, content)
